"""Tests for satip.eumetsat."""


def test_filename_to_datetime():
    """If there were a test here, there would also be a docstring here."""
    pass


def test_data_tailor():
    """If there were a test here, there would also be a docstring here."""
    pass
